@@ -20,6 +20,7 @@ except ImportError:
 
 
 _re_boolean_check_constraint = re.compile(r"(?:(?:.*?)\.)?(.*?) IN \(0, 1\)")
+_re_column_name = re.compile(r'(?:(["`])(?:.*)\1\.)?(["`]?)(.*)\2')
 _re_enum_check_constraint = re.compile(r"(?:(?:.*?)\.)?(.*?) IN \((.+)\)")
 _re_enum_item = re.compile(r"'(.*?)(?<!\\)'")
 
@@ -97,6 +98,15 @@ def _render_column_type(coltype):
     return text
 
 
+def _render_fk(fk):
+    text = "ForeignKey('%s.%s'" % (fk.column.table.fullname, fk.column.name)
+    if fk.deferrable:
+        text += ', deferrable=True'
+    if fk.initially:
+        text += ', initially=%r' % fk.initially
+    return text + ')'
+
+
 def _render_column(column, show_name):
     kwarg = []
     is_sole_pk = column.primary_key and len(column.table.primary_key) == 1
@@ -105,6 +115,9 @@ def _render_column(column, show_name):
                     for c in column.table.constraints)
     has_index = any(set(i.columns) == set([column]) for i in column.table.indexes)
 
+    # Render the column type if there are no foreign keys on it or any of them points back to itself
+    render_coltype = not dedicated_fks or any(fk.column is column for fk in dedicated_fks)
+
     if column.key != column.name:
         kwarg.append('key')
     if column.primary_key:
@@ -123,8 +136,8 @@ def _render_column(column, show_name):
 
     return 'Column({0})'.format(', '.join(
         ([repr(column.name)] if show_name else []) +
-        ([_render_column_type(column.type)] if not dedicated_fks else []) +
-        [repr(x) for x in dedicated_fks] +
+        ([_render_column_type(column.type)] if render_coltype else []) +
+        [_render_fk(x) for x in dedicated_fks] +
         [repr(x) for x in column.constraints] +
         ['{0}={1}'.format(k, repr(getattr(column, k))) for k in kwarg]))
 
@@ -132,7 +145,7 @@ def _render_column(column, show_name):
 def _render_constraint(constraint):
     if isinstance(constraint, ForeignKeyConstraint):
         local_columns = constraint.columns
-        remote_columns = ['{0}.{1}'.format(fk.column.table.name, fk.column.name)
+        remote_columns = ['{0}.{1}'.format(fk.column.table.fullname, fk.column.name)
                           for fk in constraint.elements]
         return 'ForeignKeyConstraint({0!r}, {1!r})'.format(local_columns, remote_columns)
     elif isinstance(constraint, CheckConstraint):
@@ -166,6 +179,7 @@ class Model(object):
     def __init__(self, table):
         super(Model, self).__init__()
         self.table = table
+        self.schema = table.schema
 
         # Adapt column types to the most reasonable generic types (ie. VARCHAR -> String)
         for column in table.columns:
@@ -224,6 +238,9 @@ class ModelTable(Model):
             if len(index.columns) > 1:
                 text += '    {0},\n'.format(_render_index(index))
 
+        if self.schema:
+            text += "    schema='{0}',\n".format(self.schema)
+
         return text.rstrip('\n,') + '\n)'
 
 
@@ -293,7 +310,18 @@ class ModelClass(Model):
         for index in self.table.indexes:
             if len(index.columns) > 1:
                 table_args.append(_render_index(index))
-        if table_args:
+
+        table_kwargs = {}
+        if self.schema:
+            table_kwargs['schema'] = self.schema
+
+        kwargs_items = ', '.join('{0!r}: {1!r}'.format(key, table_kwargs[key]) for key in table_kwargs)
+        kwargs_items = '{{{0}}}'.format(kwargs_items) if kwargs_items else None
+        if table_kwargs and not table_args:
+            text += '    __table_args__ = {0}\n'.format(kwargs_items)
+        elif table_args:
+            if kwargs_items:
+                table_args.append(kwargs_items)
             if len(table_args) == 1:
                 table_args[0] += ','
             text += '    __table_args__ = (\n        {0}\n    )\n'.format(',\n        '.join(table_args))
@@ -434,7 +462,7 @@ class CodeGenerator(object):
         self.models = []
         self.collector = ImportCollector()
         classes = {}
-        for table in sorted(metadata.tables.values(), key=lambda t: t.name):
+        for table in sorted(metadata.tables.values(), key=lambda t: (t.schema or '', t.name)):
             # Support for Alembic and sqlalchemy-migrate -- never expose the schema version tables
             if table.name in ('alembic_version', 'migrate_version'):
                 continue
@@ -456,7 +484,7 @@ class CodeGenerator(object):
                         # Turn any integer-like column with a CheckConstraint like "column IN (0, 1)" into a Boolean
                         match = _re_boolean_check_constraint.match(sqltext)
                         if match:
-                            colname = match.group(1).strip('`"')
+                            colname = _re_column_name.match(match.group(1)).group(3)
                             table.constraints.remove(constraint)
                             table.c[colname].type = Boolean()
                             continue
@@ -464,7 +492,7 @@ class CodeGenerator(object):
                         # Turn any string-type column with a CheckConstraint like "column IN (...)" into an Enum
                         match = _re_enum_check_constraint.match(sqltext)
                         if match:
-                            colname = match.group(1).strip('`"')
+                            colname = _re_column_name.match(match.group(1)).group(3)
                             items = match.group(2)
                             if isinstance(table.c[colname].type, String):
                                 table.constraints.remove(constraint)
