@@ -5,7 +5,8 @@ import re
 
 from nose.tools import eq_
 from sqlalchemy.engine import create_engine
-from sqlalchemy.schema import MetaData, Table, Column, CheckConstraint, UniqueConstraint, Index, ForeignKeyConstraint
+from sqlalchemy.schema import (MetaData, Table, Column, CheckConstraint, UniqueConstraint, Index, ForeignKey,
+                               ForeignKeyConstraint)
 from sqlalchemy.types import INTEGER, SMALLINT, VARCHAR, NUMERIC
 from sqlalchemy.dialects.postgresql.base import BIGINT, DOUBLE_PRECISION, BOOLEAN, ENUM
 from sqlalchemy.dialects.mysql.base import TINYINT
@@ -23,10 +24,10 @@ else:
     remove_unicode_prefixes = lambda text: text
 
 
-class TestCodeGenerator(object):
+class TestModelGenerator(object):
     def setup(self):
         self.metadata = MetaData(create_engine('sqlite:///'))
-    
+
     def generate_code(self, **kwargs):
         codegen = CodeGenerator(self.metadata, **kwargs)
         sio = StringIO()
@@ -793,3 +794,139 @@ class SimpleItems(Base):
 
     id = Column(Integer, primary_key=True)
 """)
+
+    def test_table_kwargs(self):
+        Table(
+            'simple_items', self.metadata,
+            Column('id', INTEGER, primary_key=True),
+            schema='testschema'
+        )
+
+        eq_(self.generate_code(), """\
+# coding: utf-8
+from sqlalchemy import Column, Integer
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class SimpleItem(Base):
+    __tablename__ = 'simple_items'
+    __table_args__ = {'schema': 'testschema'}
+
+    id = Column(Integer, primary_key=True)
+""")
+
+    def test_table_args_kwargs(self):
+        simple_items = Table(
+            'simple_items', self.metadata,
+            Column('id', INTEGER, primary_key=True),
+            Column('name', VARCHAR),
+            schema='testschema'
+        )
+        simple_items.indexes.add(Index('testidx', simple_items.c.id, simple_items.c.name))
+
+        eq_(self.generate_code(), """\
+# coding: utf-8
+from sqlalchemy import Column, Index, Integer, String
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class SimpleItem(Base):
+    __tablename__ = 'simple_items'
+    __table_args__ = (
+        Index('testidx', 'id', 'name'),
+        {'schema': 'testschema'}
+    )
+
+    id = Column(Integer, primary_key=True)
+    name = Column(String)
+""")
+
+    def test_schema_table(self):
+        Table(
+            'simple_items', self.metadata,
+            Column('name', VARCHAR),
+            schema='testschema'
+        )
+
+        eq_(self.generate_code(), """\
+# coding: utf-8
+from sqlalchemy import Column, MetaData, String, Table
+
+
+metadata = MetaData()
+
+
+t_simple_items = Table(
+    'simple_items', metadata,
+    Column('name', String),
+    schema='testschema'
+)
+""")
+
+    def test_foreign_key_options(self):
+        Table(
+            'simple_items', self.metadata,
+            Column('name', VARCHAR, ForeignKey('simple_items.name', deferrable=True, initially='DEFERRED'))
+        )
+
+        eq_(self.generate_code(), """\
+# coding: utf-8
+from sqlalchemy import Column, ForeignKey, MetaData, String, Table
+
+
+metadata = MetaData()
+
+
+t_simple_items = Table(
+    'simple_items', metadata,
+    Column('name', String, ForeignKey('simple_items.name', deferrable=True, initially='DEFERRED'))
+)
+""")
+
+    def test_foreign_key_schema(self):
+        Table(
+            'simple_items', self.metadata,
+            Column('id', INTEGER, primary_key=True),
+            Column('other_item_id', INTEGER),
+            ForeignKeyConstraint(['other_item_id'], ['otherschema.other_items.id'])
+        )
+        Table(
+            'other_items', self.metadata,
+            Column('id', INTEGER, primary_key=True),
+            schema='otherschema'
+        )
+
+        eq_(self.generate_code(), """\
+# coding: utf-8
+from sqlalchemy import Column, ForeignKey, Integer
+from sqlalchemy.orm import relationship
+from sqlalchemy.ext.declarative import declarative_base
+
+
+Base = declarative_base()
+metadata = Base.metadata
+
+
+class SimpleItem(Base):
+    __tablename__ = 'simple_items'
+
+    id = Column(Integer, primary_key=True)
+    other_item_id = Column(ForeignKey('otherschema.other_items.id'))
+
+    other_item = relationship('OtherItem')
+
+
+class OtherItem(Base):
+    __tablename__ = 'other_items'
+    __table_args__ = {'schema': 'otherschema'}
+
+    id = Column(Integer, primary_key=True)
+""")
